<?php
require_once 'config.php';  // Asegúrate de que la ruta al archivo de configuración es correcta.
header('Content-Type: application/json');
header('Access-Control-Allow-Origin: *');  // Permite solicitudes de cualquier origen
header('Access-Control-Allow-Methods: GET, POST');  // Especifica los métodos permitidos

header('Access-Control-Allow-Methods: GET, POST, OPTIONS');

header('Access-Control-Allow-Headers: Content-Type, Authorization');  // Especifica las cabeceras permitidas

// Conexión a la base de datos usando PDO
$options = [
    PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
    PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
    PDO::ATTR_EMULATE_PREPARES => false,
];

$dsn = "mysql:host=" . DB_HOSTNAME . ";port=" . DB_PORT . ";dbname=" . DB_DATABASE . ";charset=utf8mb4";
try {
    $pdo = new PDO($dsn, DB_USERNAME, DB_PASSWORD, $options);
} catch (PDOException $e) {
    http_response_code(500);
    echo json_encode(['error' => "Error de conexión: " . $e->getMessage()]);
    exit;
}

// Autenticación básica
if (!isset($_SERVER['PHP_AUTH_USER'])) {
    header('WWW-Authenticate: Basic realm="Mi API"');
    header('HTTP/1.0 401 Unauthorized');
    echo json_encode(['error' => 'Autenticación requerida.']);
    exit;
} else {
    if ($_SERVER['PHP_AUTH_USER'] !== 'Super' || $_SERVER['PHP_AUTH_PW'] !== 'Lmas2020!1') {
        header('HTTP/1.0 401 Unauthorized');
        echo json_encode(['error' => 'Credenciales no válidas.']);
        exit;
    }
}

$method = $_SERVER['REQUEST_METHOD'];

switch ($method) {
    case 'POST':
        // Incluir aquí la lógica para manejar POST como ya está en tu código
        break;
    case 'GET':
        if (isset($_GET['sectionid'])) {
            $sectionid = $_GET['sectionid'];
            $sql = "SELECT * FROM `oczv_optionsaved` WHERE `sectionid` = :sectionid";
            $params = ['sectionid' => $sectionid];

            if (isset($_GET['prodid'])) {
                $sql .= " AND `prodid` = :prodid";
                $params['prodid'] = $_GET['prodid'];
            }
            if (isset($_GET['optid'])) {
                $sql .= " AND `optid` = :optid";
                $params['optid'] = $_GET['optid'];
            }
            if (isset($_GET['optsessionid'])) {
                $sql .= " AND `optsessionid` = :optsessionid";
                $params['optsessionid'] = $_GET['optsessionid'];
            }
            if (isset($_GET['optionnume'])) {
                $sql .= " AND `optionnume` = :optionnume";
                $params['optionnume'] = $_GET['optionnume'];
            }

            $stmt = $pdo->prepare($sql);
            $stmt->execute($params);
            $results = $stmt->fetchAll();

            if ($results) {
                echo json_encode(['results' => $results]);
            } else {
                echo json_encode(['error' => 'No se encontraron datos.']);
            }
        } else {
            http_response_code(400);
            echo json_encode(['error' => 'Falta el parámetro sectionid.']);
        }
        break;
    default:
        http_response_code(405);
        echo json_encode(['error' => 'Método HTTP no soportado']);
        break;
}
?>
